<?php
namespace App\Service;

class ConsumerService {
    
    const GEOCODE_URL = "https://maps.google.com/maps/api/geocode/json?key=AIzaSyBfblcHNG4xb8lcwzD5sUvQ-dXIuY3QbNs";
    
    private $util;
    private $factory;

    public function __construct (\App\Util\Util $util,
                                 \App\Factory\ConsumerFactory $factory) {

        $this->util    = $util;
        $this->factory = $factory;
    }
    
    public function getConsumer($cdcliente, $cdconsumidor, $clientDetails) {
        $consumer = [];
        
        if(!empty($clientDetails["addressDetails"]) && !empty($clientDetails["addressDetails"]["complete_address"])) {
            
            $url = self::GEOCODE_URL . "&address=" . urlencode($clientDetails["addressDetails"]["complete_address"]);
            
            $address = json_decode($this->util->executeGoogleService($url, "GET", [], []), true);
            
            if (empty($address['results'][0]))
                throw new \Exception("Nao foi possivel encontrar o endereço deste pedido, endereço incompleto ou inexistente.");
            
            $consumer = $this->getConsumerAddress($address['results'][0], $clientDetails["addressDetails"]);
        } else {

            $consumer = $this->factory->getConsumer($cdcliente, $cdconsumidor);
            
            if(empty($consumer)) 
                throw new \Exception("Nao foi possivel encontrar o consumidor. CDCLIENTE: $cdcliente, CDCONSUMIDOR: $cdconsumidor");
            
        }
        
        return $consumer;
    }
    
    public function getDefaultConsumer($cdcliente, $cdconsumidor) {
        $consumer = [];
        
        $consumer = $this->factory->getConsumer($cdcliente, $cdconsumidor);
            
        if(empty($consumer)) 
            throw new \Exception("Nao foi possivel encontrar o consumidor. CDCLIENTE: $cdcliente, CDCONSUMIDOR: $cdconsumidor");
        
        
        return $consumer;
    }
    
    public function getConsumerAddress($address, $consumerAddress) {
        $dsendecons = "";
        
        if(!empty($consumerAddress["street_shorcut"]) && !empty($consumerAddress["street_name"]))
            $dsendecons = $consumerAddress["street_shorcut"] . " " .  $consumerAddress["street_name"];
        else 
            $dsendecons = explode(",", $address['formatted_address'])[0];
        
        $nrcepcons  = !empty($consumerAddress['postal_code']) ? str_replace("-", "", $consumerAddress['postal_code']) : "";
        
        $consumerData = [
            "CDBAIRRO"        => "", 
            "CDMUNICIPIO"     => "", 
            "SGESTADO"        => !empty($consumerAddress["federal_unit"]) ? strtoupper($consumerAddress["federal_unit"]) : "", 
            "NRCEPCONS"       => $nrcepcons, 
            "NMBAIRCONS"      => !empty($consumerAddress["neighborhood"]) ? $consumerAddress["neighborhood"] : "", 
            "NRENDECONS"      => !empty($consumerAddress["street_number"]) ? $consumerAddress["street_number"] : "", 
            "DSENDECONS"      => $dsendecons, 
            "DSREFERENDECONS" => !empty($consumerAddress["complement"]) ? $consumerAddress["complement"] : "", 
            "DSCOMPLENDECONS" => !empty($consumerAddress["address_description"]) ? $consumerAddress["address_description"] : "", 
            "NMMUNICIPIO"     => !empty($consumerAddress["city"]) ? $consumerAddress["city"] : ""
        ];
        
        if(!$this->validateAddressOutput($consumerData)) {
            
            foreach($address['address_components'] as $component) {
                if(array_search("administrative_area_level_1", $component['types']) !== false) 
                    $consumerData["SGESTADO"] = !empty($consumerData["SGESTADO"]) ? $consumerData["SGESTADO"] : $component['short_name']; 
    
                if(array_search("postal_code", $component['types']) !== false) 
                    $consumerData["NRCEPCONS"] = !empty($consumerData["NRCEPCONS"]) ? $consumerData["NRCEPCONS"] : str_replace("-", "", $component['short_name']);
    
                if(array_search("street_number", $component['types']) !== false) 
                    $consumerData["NRENDECONS"] = !empty($consumerData["NRENDECONS"]) ? $consumerData["NRENDECONS"] : $component['short_name'];
    
                if(array_search("administrative_area_level_2", $component['types']) !== false) 
                    $consumerData["NMMUNICIPIO"] = !empty($consumerData["NMMUNICIPIO"]) ? $consumerData["NMMUNICIPIO"] : $component['short_name'];
    
                if(array_search("sublocality_level_1", $component['types']) !== false) 
                    $consumerData["NMBAIRCONS"] = !empty($consumerData["NMBAIRCONS"]) ? $consumerData["NMBAIRCONS"] : $component['short_name'];
            }
        }
            
        $consumerData["CDMUNICIPIO"] = $this->factory->getCityCode($consumerData["SGESTADO"], $this->util->removeAcentos($consumerData["NMMUNICIPIO"]));
        $consumerData["CDBAIRRO"]    = $this->factory->getNeighborhoodCode($consumerData["SGESTADO"], $consumerData["CDMUNICIPIO"], $this->util->removeAcentos($consumerData["NMBAIRCONS"]));

        return $consumerData;
    }
    
    private function validateAddressOutput($addressOutput) {
        if( empty($addressOutput["SGESTADO"]) || empty($addressOutput["NRCEPCONS"]) || 
            empty($addressOutput["NRENDECONS"]) || empty($addressOutput["NMMUNICIPIO"]) || 
            empty($addressOutput["NMBAIRCONS"])) {
                
            return false;
        }
        return true;
    }
    
    public function getConsumerCity($consumerData) {
        
        $CDMUNICIPIO = $this->factory->getCityCode($consumerData["SGESTADO"], $this->util->removeAcentos($consumerData["NMMUNICIPIO"]));
            
        if(empty($CDMUNICIPIO)) 
            throw new \Exception("Nao foi possivel encontrar o municipio do consumidor CDCLIENTE: {$consumerData['CDCLIENTE']}, CDCONSUMIDOR: {$consumerData['CDCONSUMIDOR']}");
        
        
        return $CDMUNICIPIO;
    }
    
    public function getConsumerBairCode($consumerData) {
        
        $CDBAIRRO = $this->factory->getNeighborhoodCode($consumerData["SGESTADO"], $consumerData["CDMUNICIPIO"], $this->util->removeAcentos($consumerData["NMBAIRCONS"]));
            
        return $CDBAIRRO;
    }
}
